@@ -1,4 +1,5 @@
 import re
+import bz2
 
 # Stop Words courtesy of http://www.dcs.gla.ac.uk/idom/ir_resources/linguistic_utils/stop_words
 STOP_WORDS = r"""\b(a|about|above|across|after|afterwards|again|against|all|almost|alone|along|already|also|
@@ -80,6 +81,53 @@ class Levenshtein():
         distances.sort()
         return distances[0:number_of_matches]
 
+# NCD
+#
+# Author: Francisco Ribeiro <francisco.gtr@gmail.com>
+#
+
+def ncd(key1, key2):
+    """
+    Performs an NCD (Normalized Compression Distance) comparison between two keys.
+
+    You can read some documentation on the algorithm here: http://www.sophos.com/blogs/sophoslabs/?p=188
+
+    These can either be the contents of a file or search terms.
+
+    This will be a float value between 0.0 and 1.1, where the lower the value the
+    closer in similarity.
+
+    :key1: First value to compare, must be compressable
+    :key2: Second value to compare, must be compressable.
+
+    >>> key1 = "h3ll0"
+    >>> key2 = "hello"
+    >>> key3 = "lehlo"
+    >>> key4 = "aeiou"
+
+    >>> ncd(key1, key2)
+    0.071428571428571425
+    >>> ncd(key2, key3)
+    0.024390243902439025
+    >>> ncd(key3, key4)
+    0.11904761904761904
+    >>> ncd(key1, key3)
+    0.095238095238095233
+    >>> ncd(key1, key4)
+    0.16666666666666666
+    >>> ncd(key2, key4)
+    0.095238095238095233
+    """
+
+    key3 = key1 + key2
+    
+    ck1 = bz2.compress(key1)
+    ck2 = bz2.compress(key2)
+    ck3 = bz2.compress(key3)
+    
+    n = float((len(ck3) - min(len(ck1), len(ck2)))) / float(max(len(ck1), len(ck2)))
+
+    return n
 
 #######################################################################
 #######################################################################
@@ -446,9 +494,9 @@ class Ngram():
       This is a class method. It can be called without instantiating
       the ngram class first. Example:
 
-      >>> from ngram import ngram
-      >>> ngram.compare('sfewefsf', 'sdfafwgah')
-      >>> 0.050000000000000003
+      >> ngram = Ngram()
+      >> ngram.compare('sfewefsf', 'sdfafwgah')
+      >> 0.050000000000000003
 
       PARAMETERS:
          s1 -  First string
@@ -468,11 +516,14 @@ if __name__ == "__main__":
 
 
     # A simple example
-    base = ['sdafaf', 'asfwef', 'asdfawe', 'adfwe', 'askfjwehiuasdfji']
-    tg = ngram(base, min_sim = 0.0)
+    #base = ['sdafaf', 'asfwef', 'asdfawe', 'adfwe', 'askfjwehiuasdfji']
+    #tg = ngram(base, min_sim = 0.0)
 
-    pprint.pprint(tg.getSimilarStrings('askfjwehiuasdfji'))
-    print
-    pprint.pprint(ngram.compare('sdfeff', 'sdfeff'))
-    print
-    pprint.pprint(tg.getBestMatch('afadfwe', 2))
+    #pprint.pprint(tg.getSimilarStrings('askfjwehiuasdfji'))
+    #print
+    #pprint.pprint(ngram.compare('sdfeff', 'sdfeff'))
+    #print
+    #pprint.pprint(tg.getBestMatch('afadfwe', 2))
+
+    import doctest
+    doctest.testmod()
